import logging, os


def singleton(class_):
	instances = {}

	def getinstance(*args, **kwargs):
		if class_ not in instances:
			instances[class_] = class_(*args, **kwargs)
		return instances[class_]

	return getinstance


class Singleton(type):
	_instances = {}

	def __call__(cls, *args, **kwargs):
		if cls not in cls._instances:
			cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
		return cls._instances[cls]





import logging
import os
from pathlib import Path

from . import constants as Const
from .exceptions import PyenvException


def __get_environment_file(config_path: str, file_name: str, config_type: str):
	if not os.path.isdir(config_path):
		raise PyenvException.BaseDirDoesNotExists(config_path)

	config_file_list = list(Path(config_path).glob(f"*{config_type}"))
	local_config_file = Path(config_path).joinpath(f"{Const.LOCAL_CONFIG_FILE_PREFIX}{config_type}")
	main_config_file = Path(config_path).joinpath(f"{file_name}{config_type}")
	

	if local_config_file in config_file_list or main_config_file in config_file_list:
		if local_config_file in config_file_list:
			return local_config_file.__str__()
		else:
			return main_config_file.__str__()
	else:
		raise PyenvException.ConfigFileDoesNotExist(config_path)


def __load_configuration_file(config_file: str, config_type: str):
	if config_type == Const.DOT_ENV_FILE:
		from dotenv import load_dotenv
		return load_dotenv(config_file, verbose=True)


def load_config(env_path: str = '', file_name: str = '', file_type: str = Const.DOT_ENV_FILE):
	try:
		config_file = __get_environment_file(config_path=env_path, file_name=file_name, config_type=file_type)
		load_config = __load_configuration_file(config_file=config_file, config_type=file_type)
		if load_config:
			os.environ[Const.PYENV_LOAD_STATUS_KEY] = '1'
		else:
			raise PyenvException.ConfigurationLoadingFailed(config_file)
	except Exception as e:
		logging.error(e)
		exit()



from .config_loader import load_config
from .config import Config
from .constants import (
	DOT_ENV_FILE, LOCAL_CONFIG_FILE_PREFIX, ENVIRONMENT_VARIABLE_NAME, DEBUG_VARIABLE_NAME, DEBUG_VARIABLE_VALUE,
	PYENV_LOAD_STATUS_KEY
)

__all__ = [
	"load_config",
	"Config",
	"DOT_ENV_FILE",
	"LOCAL_CONFIG_FILE_PREFIX",
	"ENVIRONMENT_VARIABLE_NAME",
	"DEBUG_VARIABLE_VALUE",
	"DEBUG_VARIABLE_NAME",
	"PYENV_LOAD_STATUS_KEY",
]



from os import environ

from .config_var import ConfigVar
from . import constants as Const


class Config:

	@staticmethod
	def is_loaded() -> bool:
		return environ.get(Const.PYENV_LOAD_STATUS_KEY) is not None

	@staticmethod
	def get(key: str = '', default=None) -> ConfigVar:
		return ConfigVar(key=key, value=environ.get(key, default=default))

	@staticmethod
	def environment() -> str:
		return Config.get(Const.ENVIRONMENT_VARIABLE_NAME, default=None)

	@staticmethod
	def debug():
		return Config.get(Const.DEBUG_VARIABLE_NAME).as_bool(default=Const.DEBUG_VARIABLE_VALUE)

IS_CONFIG_LOADED = Config.is_loaded()



import logging, os

from . import constants as Const


class EnvUtils:

	@staticmethod
	def read_file(file: str = None):
		with open(file, 'r+') as _file:
			return _file.read()

	@staticmethod
	def write_to_file(file: str = None, data: str = ''):
		with open(file, 'w+') as _file:
			_file.write(data)

	@staticmethod
	def build_main_config_file(main_config_file: str = '', local_config_file: str = ''):
		try:
			import re
			EnvUtils.write_to_file(
				main_config_file,
				re.compile(Const.REGEX_CONFIG_VARIABLE_SELECT_PATTERN).sub(
					repl=Const.REGEX_CONFIG_VARIABLE_REPLACEMENT_PATTERN,
					string=EnvUtils.read_file(local_config_file)
				)
			)
		except Exception as e:
			logging.exception(e)

	@staticmethod
	def make_dir(directory: str, mode=0o777):
		try:
			if not os.path.isdir(directory):
				os.mkdir(directory, mode=mode)
		except Exception as e:
			logging.error(f"Exception occurred while creating directory[{directory}] :\n {e}")



import re

PYENV_LOAD_STATUS_KEY = 'PYENV_CONFIG_LOADED'

DOT_ENV_FILE = ".env"

LOCAL_CONFIG_FILE_PREFIX = "local"

ENVIRONMENT_VARIABLE_NAME = "ENVIRONMENT"

DEBUG_VARIABLE_NAME = "ENVIRONMENT"

DEBUG_VARIABLE_VALUE = False

REGEX_CONFIG_VARIABLE_SELECT_PATTERN = r"(\r\n|\r|\n|\s)*?([\w_]*)=(['\"])?([\w\d()\-:/*?=@.+!%$_#^&,]*)(['\"])?([\s\r\n]*)"

REGEX_CONFIG_VARIABLE_REPLACEMENT_PATTERN = r'\1\2=\3\2_VALUE\5\6'



from setuptools import setup, find_packages

long_desc = ''
with open('README.md') as readme_file:
	long_desc = readme_file.read()

setup(
	name='pyutilbox',
	version='0.1',
	license='MIT',
	author='Prakash Leon Mishra',
	author_email='dev.leonmishra@gmail.com',
	description='A simple Package to load & parse configuration to diffrent datatypes',
	long_description=long_desc,
	long_description_content_type="text/markdown",
	keywords='Python _pyconfig loader, django configuration loader, flask config loader',
	package_dir={"": "src"},
	packages=find_packages(where="src"),
	python_requires=">=3.0",
	url='https://github.com/rionzz-inc/pyutilbox',
	# download_url='https://github.com/user/reponame/archive/v_01.tar.gz',  # I explain this
	# later on

	install_requires=[
		'python-dotenv',
	],
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	]

)

if __name__ == "__main__":
	print (find_packages(where="src"))


class BaseExceptions(Exception):
	_message: str = None
	_errors: list = []

	def __init__(self, *errors):
		errors = '\n Ref : '.join(errors)
		super().__init__('Exception Occurred.' if self._message is None else f"\n{self._message} :\n{errors}")
		self._errors = errors if isinstance(errors, dict) else list()

	@property
	def message(self):
		return self._message

	@property
	def error_list(self):
		return self._errors


class PyenvException():
	class BaseDirDoesNotExists(BaseExceptions):
		_message = "Root Directory Does Not Exists."

	class ConfigFileDoesNotExist(BaseExceptions):
		_message = "Configuration File Does Not Exists."

	class ConfigurationLoadingFailed(BaseExceptions):
		_message = "Failed To Load Configuration From Config File."
